@@ -1,5 +1,5 @@
 ;;  -*-  indent-tabs-mode:nil; coding: utf-8 -*-
-;;  Copyright (C) 2014,2015,2017
+;;  Copyright (C) 2014,2015,2017,2018
 ;;      "Mu Lei" known as "NalaGinrut" <NalaGinrut@gmail.com>
 ;;  Artanis is free software: you can redistribute it and/or modify
 ;;  it under the terms of the GNU General Public License and GNU
@@ -86,7 +86,7 @@
 
 ;; Use Cases
 ;; A. Static page
-;; Nothing to say. 
+;; Nothing to say.
 
 ;; B. Dynamic page
 ;; 1. The developer must assess how heavily it can be cached and what the
@@ -144,7 +144,7 @@
          `(public ,(cons 'max-age m))))
       (('private . maxage)
        (let ((m (if (null? maxage) (get-conf '(cache maxage)) (car maxage))))
-           `(private ,(cons 'max-age m))))
+         `(private ,(cons 'max-age m))))
       (else (throw 'artanis-err "->cc: Invalid opts!" o))))
   (cache-to-tlb! rc etag) ; cache the hash the TLB
   (response-emit body #:headers `((ETag . ,etag)
@@ -155,7 +155,7 @@
    ((file-exists? filename)
     (let ((st (stat filename)))
       ;; NOTE: ETag must be around with double-quote explicitly!
-      (format #f "\"~X-~X-~X\"" 
+      (format #f "\"~X-~X-~X\""
               (stat:ino st) (stat:mtime st) (stat:size st))))
    (else '())))
 
@@ -164,7 +164,7 @@
 
 (define-syntax-rule (->If-None-Match rc)
   (assoc-ref (->headers rc) 'if-none-match))
-  
+
 (define (If-None-Match-hit? rc etag)
   (define (-> e)
     ;; NOTE: The if-none-match sent from Chromium dropped double-quote,
@@ -178,12 +178,12 @@
 
 ;; ETag for dynamic content is content based
 (define (try-to-cache-body rc body . opts)
-  (define (gen-etag-for-dynamic-content b)
+  (define (gen-etag-for-dynamic-content)
     ;; NOTE: ETag must be around with double-quote explicitly!
-    (string-concatenate (list "\"" (string->md5 b) "\"")))
+    (string-concatenate (list "\"" (string->md5 body) "\"")))
   (define-syntax-rule (get-proper-hash)
     (or (get-from-tlb (rc-path rc)) ; get hash from TLB
-        (gen-etag-for-dynamic-content body))) ; or generate new hash
+        (gen-etag-for-dynamic-content))) ; or generate new hash
   (cond
    ((cacheable-request? (rc-req rc))
     ;; NOTE: In Artanis, dynamic page is content based caching, so we don't checkout
@@ -226,6 +226,6 @@
              ((? integer? m) m)
              (((? integer? m)) m)
              (() (get-conf '(cache maxage)))
-             (else (throw 'artanis-err "->maxage: Invalid maxage!" maxage)))))
+             (else (throw 'artanis-err ->maxage "Invalid maxage!" maxage)))))
     (DEBUG "Cache maxage is ~a~%" m)
     m))
