@@ -61,9 +61,9 @@
 (define (->mysql mysql)
   (match mysql
     (($ <mysql> ($ <db> _ username passwd) dbname addr #f)
-     (format #f "~a:~a::tcp:~a" username passwd addr))
+     (format #f "~a:~a:~a:tcp:~a" username passwd dbname addr))
     (($ <mysql> ($ <db> _ username passwd) dbname #f socketfile)
-     (format #f "~a:~a::socket:~a" username passwd socketfile))
+     (format #f "~a:~a:~a:socket:~a" username passwd dbname socketfile))
     (else (error 'mysql "Wrong connection config!" mysql))))
 
 (define-record-type <sqlite3>
@@ -125,16 +125,16 @@
 ;; e.g: (connect-db "mysql" "root:123:artanis:tcp:localhost:3306")
 (define connect-db
   (case-lambda*
-   ((dbd str) (%do-connect dbd str))
-   ((dbd #:key (db-name "artanis") (db-username "root") (db-passwd "")
-         (addr "localhost:3306") (proto 'tcp))
-    (let ((str (case dbd
-                 ((mysql) (format #f "~a:~a:~a:~a:~a"
-                                  db-username db-passwd db-name proto addr))
-                 ((postgresql) (format #f "~a:~a:~a:~a:~a"
-                                       db-username db-passwd db-name proto addr))
-                 ((sqlite3) (format #f "~a" db-name)))))
-      (%do-connect dbd str)))))
+      ((dbd str) (%do-connect dbd str))
+    ((dbd #:key (db-name "artanis") (db-username "root") (db-passwd "")
+          (addr "localhost:3306") (proto 'tcp))
+     (let ((str (case dbd
+                  ((mysql) (format #f "~a:~a:~a:~a:~a"
+                                   db-username db-passwd db-name proto addr))
+                  ((postgresql) (format #f "~a:~a:~a:~a:~a"
+                                        db-username db-passwd db-name proto addr))
+                  ((sqlite3) (format #f "~a" db-name)))))
+       (%do-connect dbd str)))))
 
 (define (new-DB)
   ;; TODO:
@@ -157,14 +157,12 @@
       (else (error new-DB "Something is wrong, invalid db!" db)))))
 
 (define (create-new-DB-conn)
-  (let ((conn (DB-do-conn! (new-DB)))
-        (dbd (get-conf '(db dbd)))
-        (db-name (get-conf '(db name))))
+  (let ((conn (DB-do-conn! (new-DB))))
     (when (not (db-conn-success? conn))
-      (error init-connection-pool "Database connect failed: "
+      (throw 'artanis-err 500 create-new-DB-conn
+             "Database connect failed: "
              (db-conn-failed-reason conn)))
-    (when (not (eq? dbd 'sqlite3)) ; sqlite3 doesn't support 'USE' statement
-      (DB-query conn (->sql use db-name)))
+    (run-hook *DB-conn-init-hook* conn)
     conn))
 
 (define (get-conn-from-pool)
@@ -196,6 +194,23 @@
 (define (db-conn-success? conn)
   (zero? (%db-conn-stat conn car)))
 
+(define (db-conn-is-closed? conn)
+  ;; NOTE: This feature requires guile-dbd-mysql-2.1.7 or higher
+  ;; https://github.com/opencog/guile-dbi
+  (%db-conn-stat conn (lambda (s)
+                        (case (get-conf '(db dbd))
+                          ((mysql)
+                           (= (car s) 2006))
+                          ((postgresql)
+                           ;; TODO
+                           #f)
+                          ((sqlite3)
+                           ;; TODO
+                           #f)
+                          (else
+                           (throw 'artanis-err 500 db-conn-is-closed?
+                                  "Unsupported DBD `~a'" (get-conf '(db dbd))))))))
+
 (define (db-conn-failed-reason conn)
   (%db-conn-stat conn cdr))
 
@@ -214,10 +229,13 @@
 ;; ---------------------conn operations-------------------------------
 ;; Actually, it's not `open', but get a conn from pool.
 (define (DB-open rc)
-  (let ((conn (get-conn-from-pool)))
-    (<connection>-status-set! conn 'open)
-    (rc-conn! rc conn)
-    conn))
+  (let ((cur-conn (rc-conn rc)))
+    (if cur-conn
+        cur-conn
+        (let ((conn (get-conn-from-pool)))
+          (<connection>-status-set! conn 'open)
+          (rc-conn! rc conn)
+          conn))))
 
 (define (db-query-debug-info sql)
   (when (get-conf 'debug-mode)
@@ -239,14 +257,21 @@
    (else
     (db-query-debug-info sql)
     (dbi-query (<connection>-conn conn) sql)
-    (when (not (db-conn-success? conn))
-      (format (current-error-port) "SQL: ~a~%" sql)
-      (if check?
-          (format (current-error-port) "DB-query check failed: ~a"
-                  (db-conn-failed-reason conn))
-          (throw 'artanis-err 500 DB-query "failed reason: `~a'~%"
-                 (db-conn-failed-reason conn))))
-    conn)))
+    (cond
+     ((db-conn-success? conn)
+      conn)
+     (else
+      (cond
+       ((db-conn-is-closed? conn)
+        (DEBUG "DB connection was closed, reconnect it ...~%")
+        (DB-query (create-new-DB-conn) sql #:check? check?))
+       (else
+        (format (current-error-port) "SQL: ~a~%" sql)
+        (if check?
+            (format (current-error-port) "DB-query check failed: ~a"
+                    (db-conn-failed-reason conn))
+            (throw 'artanis-err 500 DB-query "failed reason: `~a'~%"
+                   (db-conn-failed-reason conn))))))))))
 
 ;; NOTE: actually it'll never close the connection, just recycle it.
 (define (DB-close conn)
