@@ -38,6 +38,7 @@
   #:use-module (artanis third-party csv)
   #:use-module (artanis server scheduler)
   #:use-module (artanis irregex)
+  #:use-module (artanis lpc)
   #:use-module (ice-9 match)
   #:use-module (srfi srfi-1)
   #:use-module (srfi srfi-9)
@@ -67,7 +68,8 @@
              :auth
              :session
              :from-post
-             :websocket))
+             :websocket
+             :lpc))
 
 (define (define-handler method raw-rule opts-and-handler)
   (let* ((rule (string-trim-right raw-rule #\/))
@@ -143,7 +145,7 @@
         ('top (DB-get-top-row r))
         ((? positive? n) (DB-get-n-rows r n))
         (else (throw 'artanis-err 500 raw-sql-maker
-                     "Invalid mode!" mode))))))
+                     "Invalid mode `~a'!" mode))))))
 
 ;; for #:cache
 (define (cache-maker pattern rule keys)
@@ -151,10 +153,14 @@
   (define-syntax-rule (try-public rc) (if (auth-enabled? rc) 'private 'public))
   (match pattern
     ;; disable cache explicitly, sometimes users want to make sure there's no any cache.
-    ((#f) non-cache)
-    (('static maxage ...)
-     (lambda (rc)
-       (try-to-cache-static-file rc (static-filename (rc-path rc)) (try-public rc) (->maxage maxage))))
+    (#f non-cache)
+    (('static maxage0 ...)
+     (lambda* (rc #:key (dir #f) (maxage (->maxage maxage0)))
+       (let ((filename (if dir
+                           (format #f "~a/~a" dir (rc-path rc))
+                           (static-filename (rc-path rc)))))
+         (try-to-cache-static-file rc (static-filename (rc-path rc))
+                                   (try-public rc) maxage))))
     (((? string? file) maxage0 ...)
      (lambda* (rc #:key (maxage (->maxage maxage0)))
        (try-to-cache-static-file rc file (try-public rc) maxage)))
@@ -174,51 +180,53 @@
 ;;       Like crypto cookies.
 (define (cookies-maker mode rule keys)
   (define *the-remove-expires* "Thu, 01-Jan-70 00:00:01 GMT")
-  (define %cookie-set! cookie-set!)
-  (define %cookie-ref cookie-ref)
-  (define %cookie-modify cookie-modify)
-  (define %new-cookie new-cookie)
-  (define ckl '())
-  (define (cget ck)
+  (define current-cset! (make-parameter cookie-set!))
+  (define current-cref (make-parameter cookie-ref))
+  (define current-setattr (make-parameter cookie-modify))
+  (define current-maker (make-parameter new-cookie))
+  (define (cget ckl ck)
     (or (assoc-ref ckl ck)
         (throw 'artansi-err 500 cget "Undefined cookie name" ck)))
-  (define (cset! ck k v)
-    (and=> (cget ck) (cut %cookie-set! <> k v)))
-  (define (cref ck k)
-    (and=> (cget ck) (cut %cookie-ref <> k)))
-  (define (update rc)
+  (define (cset! ckl ck k v)
+    (and=> (cget ckl ck) (cut (current-cset!) <> k v)))
+  (define (cref ckl ck k)
+    (and=> (cget ckl ck) (cut (current-cref) <> k)))
+  (define (update ckl rc)
     (rc-set-cookie! rc (map cdr ckl)))
-  (define (setattr ck . kargs)
-    (apply %cookie-modify (cget ck) kargs))
+  (define (setattr ckl ck . kargs)
+    (apply (current-setattr) (cget ckl ck) kargs))
   (define (remove rc k)
     (let ((cookies (rc-set-cookie rc)))
       (rc-set-cookie!
        rc
-       `(,@cookies ,(%new-cookie #:npv '((key "")) #:expires *the-remove-expires*)))))
+       `(,@cookies ,((current-maker) #:npv '((key "")) #:expires *the-remove-expires*)))))
   (define-syntax-rule (init-cookies names)
-    (for-each
-     (lambda (ck)
-       (set! ckl (cons (cons ck (new-cookie)) ckl)))
-     names))
-  (match mode
-    (('names names ...) (init-cookies names))
-    (('custom (names ...) maker setter getter modifier)
-     (set! %new-cookie maker)
-     (set! %cookie-set! setter)
-     (set! %cookie-ref getter)
-     (set! %cookie-modify modifier)
-     (init-cookies names))
-    (else (throw 'artansi-err 500 cookies-maker
-                 "Invalid mode while init!" mode)))
-  (lambda (rc op)
+    (fold (lambda (ck p)
+            (cons (cons ck ((current-maker))) p))
+          '()
+          names))
+  (define (cookie-handler ckl rc op)
     (case op
-      ((set) cset!)
-      ((ref) cref)
-      ((setattr) setattr)
-      ((update) update)
+      ((set) (cut cset! ckl <> <> <>))
+      ((ref) (cut cref ckl <> <>))
+      ((setattr) (cut setattr ckl <> <...>))
+      ((update) (cut update ckl <>))
       ((remove) remove)
       (else (throw 'artanis-err 500 cookies-maker
-                   "Invalid operation!" op)))))
+                   "Invalid operation!" op))))
+  (match mode
+    (((or 'new 'names) names ...)
+     (let ((ckl (init-cookies names)))
+       (cut cookie-handler ckl <> <>)))
+    (('custom (names ...) maker setter getter modifier)
+     (parameterize ((current-maker maker)
+                    (current-cset! setter)
+                    (current-cref getter)
+                    (current-setattr modifier))
+       (let ((ckl (init-cookies names)))
+         (cut cookie-handler ckl <> <>))))
+    (else (throw 'artansi-err 500 cookies-maker
+                 "Invalid mode while init!" mode))))
 
 ;; for #:mime
 (define (mime-maker type rule keys)
@@ -229,38 +237,48 @@
       ((json jsonp) (lambda (args) (-> ->json-string args)))
       ((xml) (lambda (args) (-> sxml->xml-string args)))
       ((csv) (lambda (args) (-> sxml->csv-string args)))
-      ((sxml) (lambda (args) (values (object->string (car args)) #:pre-headers headers)))
-      (else (throw 'artanis-err 500 mime-maker "Invalid type!" type))))
+      ((sxml) (lambda (args) (-> object->string args)))
+      (else (throw 'artanis-err 500 mime-maker "Invalid type: ~a!" type))))
   (lambda (rc . args) (gen-mime args)))
 
 ;; for #:session
 (define (session-maker mode rule keys)
-  (define* (%spawn rc #:key (idname "sid") (proc session-spawn))
+  (define* (%spawn rc #:key (idname "sid") (proc session-spawn)
+                   (path "/") domain (expires 3600) secure (http-only #t))
     (call-with-values
-        (lambda () (proc rc))
+        (lambda () (proc rc #:expires expires))
       (lambda (sid session)
-        (let ((cookie (new-cookie #:npv `((,idname . ,sid)))))
+        (let ((cookie (new-cookie #:npv `((,idname . ,sid))
+                                  #:path "/" #:domain domain
+                                  #:expires expires #:secure secure
+                                  #:http-only http-only)))
           (and cookie (rc-set-cookie! rc (list cookie)))
           sid))))
   (define spawn-handler
     (match mode
       ((or #t 'spawn) %spawn)
-      (`(spawn ,sid) (lambda (rc) (%spawn rc #:idname sid)))
-      (`(spawn ,sid ,proc) (lambda (rc) (%spawn rc #:idname sid #:proc proc)))
+      (`(spawn ,sid) (cut %spawn <> #:idname sid <...>))
+      (`(spawn ,sid ,proc) (cut %spawn <> #:idname sid #:proc proc <...>))
       (else (throw 'artanis-err 500 session-maker "Invalid config mode: ~a" mode))))
   (define (check-it rc idname)
-    (let ((sid (any (lambda (c) (cookie-ref c idname)) (rc-cookie rc))))
-      (and=> (session-restore (or sid ""))
-             (lambda (s) (session-from-correct-client? s rc)))))
-  (lambda (rc cmd)
+    (let* ((sid (any (lambda (c) (and=> (cookie-ref c idname) car)) (rc-cookie rc)))
+           (session (session-restore (or sid ""))))
+      (case session
+        ((expired)
+         (:cookies-remove! rc idname)
+         #f)
+        ((not-found) #f)
+        (else
+         (session-from-correct-client? session rc)))))
+  (lambda (rc cmd . args)
     (match cmd
       ('check (check-it rc "sid"))
       (`(check ,sid) (check-it rc sid))
       ('check-and-spawn
-       (or (check-it rc "sid") (spawn-handler rc)))
+       (or (check-it rc "sid") (apply spawn-handler rc args)))
       (`(check-and-spawn ,sid)
-       (or (check-it rc sid) (spawn-handler rc)))
-      ('spawn (spawn-handler rc))
+       (or (check-it rc sid) (apply spawn-handler rc args)))
+      ('spawn (apply spawn-handler rc args))
       (else (throw 'artanis-err 500 session-maker "Invalid call cmd: ~a" cmd)))))
 
 ;; for #:from-post
@@ -295,9 +313,17 @@
        (throw 'artanis-err 500 store-the-bv
               "Impossible status! please report bug!"))))
   (define (post-ref plst key) (and=> (assoc-ref plst key) car))
+  (define (return-json-mapping j)
+    (let ((jt (if (bytevector? j)
+                  (json-string->scm (utf8->string j))
+                  (throw 'artanis-err 400 return-json-mapping
+                         "Invalid body type ~a~%" (detect-type-name j)))))
+      (lambda (k)
+        (hash-ref jt k))))
   (define (post-handler rc)
     (match mode
       ((or #t 'query-string 'qstr) (post->qstr-table rc))
+      ('json (return-json-mapping (rc-body rc)))
       ('qstr-safe (post->qstr-table rc 'safe))
       ((or 'bv 'bytevector) (rc-body rc))
       ;; Get mfds and handle it by yourself
@@ -328,6 +354,15 @@
 ;;
 (define (websocket-maker mode rule keys)
   (match mode
+    ('send-only
+     ;; NOTE: send-only is used for sending messages to registed websocket connection by
+     ;;       the specified pipe name. If you use this option, then the rule will not init
+     ;;       a websocket handshake. It's just for sending messages.
+     ;; NOTE; If you use this option, you can only send messages, but other can't send
+     ;;       message to you. That is to say, it's one-way transimission.
+     ;; NOTE: Other options could be bi-direction transmission without specified 'send/recv
+     ;;       explicitly.
+     (DEBUG "~a is registered to be named-pipe send only rule!" rule))
     (((or #t 'raw))
      (this-rule-enabled-websocket! rule 'raw))
     (('proto (? symbol? proto))
@@ -356,12 +391,37 @@
      (this-rule-enabled-websocket! rule 'proxy))
     (else (throw 'artanis-err 500 websocket-maker "Invalid type `~a'!" mode)))
   (lambda (rc . cmd)
-    ;; NOTE: :websocket is not for read/write, it's only used for changing status.
     (match cmd
       ('(payload) (websocket-frame-payload (rc-body rc)))
       ('(frame) (rc-body rc))
+      (`(send ,name ,data) (send-to-websocket-named-pipe name data))
       (else (throw 'artanis-err 500 websocket-maker "Invalid cmd `~a'!" cmd)))))
 
+;; for #:lpc
+;; Local Persistent Cache
+(define (lpc-maker mode rule keys)
+  (define (gen-lpc-handler rc cmd)
+    (let ((lpc (rc-lpc rc)))
+      (match cmd
+        (`(set ,key ,val) (lpc-set! lpc key val))
+        (((or 'ref 'get) key) (lpc-ref lpc key))
+        (else (throw 'artanis-err 500 gen-lpc-handler "Invalid cmd `~a'!" cmd)))))
+  (match mode
+    (#t
+     (lambda (rc . cmd)
+       (when (not (rc-lpc rc))
+         (let ((lpc (or (get-lpc-instance!) ((new-lpc)))))
+           (rc-lpc! rc lpc)))
+       (gen-lpc-handler rc cmd)))
+    ((backend . args)
+     (lambda (rc . cmd)
+       (when (not (rc-lpc rc))
+         (let ((lpc (or (get-lpc-instance!)
+                        (apply (new-lpc #:backend backend) args))))
+           (rc-lpc! rc lpc)))
+       (gen-lpc-handler rc cmd)))
+    (else (throw 'artanis-err 500 lpc-maker "Invalid pattern `~a'!" mode))))
+
 ;; ---------------------------------------------------------------------------------
 
 ;; NOTE: these short-cut-maker should never be exported!
@@ -508,7 +568,16 @@
     ;;       No protocol handling at all, just redirecting the data without cooking.
     ;;    b. cooked proxy
     ;;       There's a bound protocol instance for it.
-    (#:websocket . ,websocket-maker)))
+    (#:websocket . ,websocket-maker)
+
+    ;; Apply an instance of Local Persistent Cache
+    ;; This is useful when you want to store some key-value stuffs.
+    ;; There're 2 cases:
+    ;; 1. #t
+    ;;    Initialize lpc with default configuration, the backend is Redis, write-able.
+    ;; 2. (backend . args)
+    ;;    Specify the avalailable backend, and the args as the configuare options.
+    (#:lpc . ,lpc-maker)))
 
 (define-macro (meta-handler-register what)
   `(define-syntax-rule (,(symbol-append ': what) rc args ...)
@@ -527,6 +596,7 @@
 (meta-handler-register session)
 (meta-handler-register from-post)
 (meta-handler-register websocket)
+(meta-handler-register lpc)
 
 (define-syntax-rule (:cookies-set! rc ck k v)
   ((:cookies rc 'set) ck k v))
